﻿#=============================================================================#
# PowerShell Module - UpgradeCompatibility for mentaining compatibility when  #
# upgrading from 16.0.62 (or earlier) to 16.1.25 (or later).                  #
# Copyright (c) coolOrange s.r.l. - All rights reserved.                      #
#                                                                             #
# THIS SCRIPT/CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER   #
# EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES #
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  #
#=============================================================================#

function PrepareEnvironment()
{
	Add-Log "<OBSOLETE:  The 'PrepareEnvironment' cmdlet is obsolete and will removed in the next version>"
	$global:SYSPROPS = Get-VaultPropertyDefinitions -System
	$logger = [log4net.LogManager].GetMethod("GetLogger", [type]).Invoke($null, [coolOrange.PowerJobs.Log4NetEnvironment])
	Add-Member -InputObject $powerJobs -Name "Log" -Value $logger -MemberType NoteProperty
}

function PrepareEnvironmentForFile($testFileName)
{
	PrepareEnvironment
	if($IAmRunningInJobProcessor -eq $true) {
        if($powerJobs.Job.Params.ContainsKey("FileId"))
        {
             $fileWithCorrectVersion = (Get-VaultFile -FileId $powerJobs.Job.Params["FileId"])
             $file = Get-VaultFile -FileId $fileWithCorrectVersion.MasterId
        }
    } else {
		$file = Get-VaultFile -Properties @{$SYSPROPS.Name = $testFileName}
		if(!$file)
		{ 
			throw ("File cannot be found") 
		}
	}
	return $file
}

function PrepareEnvironmentForFolder($testFolderName)
{
	PrepareEnvironment
	if( !$IAmRunningInJobProcessor) 
	{
		$folder = $vault.DocumentService.GetFolderByPath($testFolderName)
		if(!$folder)
		{ 
			throw ("Folder cannot be found") 
		}
	}	
	return $folder
}

function Get-VaultPropertyDefinitions {
param(
	[string]$entityClassType = $null,
	[SWITCH]$System,
	[Autodesk.DataManagement.Client.Framework.Vault.Currency.Properties.PropertyDefinitionFilter] $filter =
		[Autodesk.DataManagement.Client.Framework.Vault.Currency.Properties.PropertyDefinitionFilter]::IncludeAll
)
	Add-Log "<OBSOLETE:  The '`$SysProps' object is obsolete and will removed in the next version>"
	if($System.ToBool()) {
		$filter = [Autodesk.DataManagement.Client.Framework.Vault.Currency.Properties.PropertyDefinitionFilter]::IncludeSystem -bor
			[Autodesk.DataManagement.Client.Framework.Vault.Currency.Properties.PropertyDefinitionFilter]::IncludeInternal
	}
	$allPropDefs = $vaultConnection.PropertyManager.GetPropertyDefinitions($entityClassType, $null, $filter)
	$propDefs = New-Object PSObject
	foreach( $p in $allPropDefs.Values ) {
	  $propDef = [PsObject] $p.DisplayName
	  $p | Get-Member -membertype properties | foreach {
	  	$propDef | add-member Noteproperty $_.Name  $p.($_.Name)
	  }
	  $propDefs | add-member Noteproperty $p.SystemName  $propDef
	}
	return $propDefs
}

function Save-FileAs {
param (
    [Parameter(Mandatory=$True,Position=1)]
	[PSCustomObject]$file,
	[Parameter(Mandatory=$True,Position=2)]
	[string] $localDestFile
)
	Add-Log "<OBSOLETE:  The 'Save-FileAs' cmdlet is obsolete and will removed in the next version>"
	$fromExtension = $file._Extension.ToUpper()
	$global:publishResult = $false;
	Create-Directory $localDestFile
	Remove-File $localDestFile
	Add-Log "Publishing from '$($file._Extension)' to '$localDestFile' ..."

	Try {
		For-TheFormat '.PDF' -ofFile $localDestFile {
			$global:publishResult = SaveAs_PDF $file $localDestFile
		}
		For-EachFormat @('.DWF','.DWFX') -ofFile $localDestFile {
			$global:publishResult = SaveAs_DWF $file $localDestFile
		}
		For-TheFormat '.DXF' -ofFile $localDestFile {
			$global:publishResult = SaveAs_DXF $file $localDestFile
		}
		For-TheFormat '.DWG' -ofFile $localDestFile {
			$global:publishResult = SaveAs_DWG $file $localDestFile
		}
        For-TheFormat '.GIF' -ofFile $localDestFile {
			$global:publishResult = SaveAs_GIF $file $localDestFile
		}
		For-EachFormat @('.IGS','.IGES') -ofFile $localDestFile {
			$global:publishResult = SaveAs_IGES $file $localDestFile
		}
		For-EachFormat @('.STP','.STEP') -ofFile $localDestFile {
			$global:publishResult = SaveAs_STEP $file $localDestFile
		}
		For-EachFormat @('.JPG','.BMP','.TIFF','.PNG') -ofFile $localDestFile {
			$global:publishResult = SaveAs_OtherFormat $file $localDestFile
	    }
	} Catch	{
		Add-Log "Failed to publish the file : $($_.Exception.Message)"
	}
	return $global:publishResult
}

function SaveAs_PDF($file, $localDestFile)
{
	Is-SupportedForFormats $file @('IAM','IPT','IDW','DWG')
    if($file._Provider -like '*Inventor*') {
        if(@('IDW','DWG') -contains $file._Extension.ToUpper()) {
            $configFile = "$($env:POWERJOBS_MODULESDIR)Export\PDF_2D.ini"
        }
    } else {
        $configFile = "$($env:POWERJOBS_MODULESDIR)Export\PDF.dwg" 
    }
	return Publish-VaultFile -Format 'PDF' -File $file -ToFile $localDestFile -Options $configFile
}

function SaveAs_DWF($file, $localDestFile)
{
	Is-SupportedForFormats $file @('IAM','IPT','IDW','DWG')
	$toExtension = [System.IO.Path]::GetExtension($localDestFile).ToUpper()
    if($file._Provider -like '*Inventor*') {
        $options =@{}
    } else {
        $options = "$($env:POWERJOBS_MODULESDIR)Export\DWF.dwg" 
    }
	return Publish-VaultFile -Format $toExtension.TrimStart(".") -File $file -ToFile $localDestFile -Options $options
}

function SaveAs_DXF($file, $localDestFile)
{
	$fromExtension = $file._Extension.ToUpper()
	if( $fromExtension -eq 'IPT' ) {
		return SaveAs_SheetDXF $file $localDestFile
	}
	Is-SupportedForFormats $file @('DWG','IDW')
	return Publish-VaultFile -Format "DXF" -File $file -ToFile $localDestFile -Options "$($env:POWERJOBS_MODULESDIR)Export\DXF_2D.ini" 
}

function SaveAs_SheetDXF()
{
	Is-SupportedForFormats $file @('IPT')
	return Publish-VaultFile -Format "DXF" -File $file -ToFile $localDestFile -Options "$($env:POWERJOBS_MODULESDIR)Export\DXF_SheetMetal.ini" 
}

function SaveAs_DWG($file, $localDestFile)
{
	Is-SupportedForFormats $file @('IAM','IPT','IDW')
    if(@('IDW','DWG') -contains $file._Extension.ToUpper() ) {
        $configFile = "$($env:POWERJOBS_MODULESDIR)Export\DWG_2D.ini" 
    } elseif ($file._Extension.ToUpper() -eq 'IPT') {
        $configFile = "$($env:POWERJOBS_MODULESDIR)Export\DWG_SheetMetal.ini" 
    } else { 
        $configFile = "$($env:POWERJOBS_MODULESDIR)Export\DWG_3D.ini" 
    }
	return Publish-VaultFile -Format "DWG" -File $file -ToFile $localDestFile -Options $configFile
}

function SaveAs_IGES($file, $localDestFile)
{
	Is-SupportedForFormats $file @('IAM','IPT')
	return Publish-VaultFile  -Format "IGES" -File $file -ToFile $localDestFile -Options "$($env:POWERJOBS_MODULESDIR)Export\IGES.ini"
}

function SaveAs_STEP($file, $localDestFile)
{
	Is-SupportedForFormats $file @('IAM','IPT')
	return Publish-VaultFile -Format "STEP" -File $file -ToFile $localDestFile -Options "$($env:POWERJOBS_MODULESDIR)Export\STEP.ini"
}

function SaveAs_GIF($file, $localDestFile)
{
	Is-SupportedForFormats $file @('IAM','IPT','IDW','DWG')
	return Publish-VaultFile -Format "GIF" -File $file -ToFile $localDestFile
}

function SaveAs_OtherFormat($file, $localDestFile)
{
	Is-SupportedForFormats $file @('IAM','IPT','IDW','DWG')
    $toExtension = [System.IO.Path]::GetExtension($localDestFile).ToUpper()
	return Publish-VaultFile -Format $toExtension.TrimStart('.') -File $file -ToFile $localDestFile
}

#region Internals

function Publish-VaultFile {
PARAM(
[Parameter(Mandatory=$True,Position=1)]
[PSCustomObject]$File,
[STRING]$ToFile,
[STRING]$Format="PDF",
$Options=$null,
[ScriptBlock]$OnExport = $null
)
    $workspace =[guid]::NewGuid().ToString("B")
    $fastOpen = $file._Extension -eq "idw" -or $file._Extension -eq "dwg" -and $file._ReleasedRevision -and $file._Provider -like '*Inventor*'
    $downloadedFiles = Save-VaultFile -File $file._FullPath -DownloadDirectory "C:\Temp\$workspace" -ExcludeChildren:$fastOpen -ExcludeLibraryContents:$fastOpen
    $file = $downloadedFiles | select -First 1
    
    try {
        Open-Document -LocalFile $file.LocalPath | where {-not $_ } | foreach {throw("Failed to open document! Error: $($_.ErrorMessage)")}
        try {
            Export-Document -Format $Format -To $ToFile -Options $Options -OnExport $OnExport  | where {-not $_ } | foreach {throw("Failed to export document! Error: $($_.ErrorMessage)")}
        } finally {
            Close-Document | where {-not $_ } | foreach {throw("Failed to close document! Error: $($_.ErrorMessage)")}
        }
    }finally{
        Clean-Up -folder "C:\Temp\$workspace"
    }
    return $true
}

function For-TheFormat {
    param (
        [string] $format = $(throw "The parameter -format is required."),
		[string] $ofFile = $(throw "The parameter -ofFile is required."),
        [ScriptBlock] $scriptBlock = $(throw "The parameter -scriptBlock is required.")
    )
	For-EachFormat @($format) $ofFile $scriptBlock
}

function For-EachFormat {
    param (
        [string[]] $formats = $(throw "The parameter -formats is required."),
		[string] $ofFile = $(throw "The parameter -ofFile is required."),
        [ScriptBlock] $scriptBlock = $(throw "The parameter -scriptBlock is required.")
    )
	$toExtension = [System.IO.Path]::GetExtension($ofFile).ToUpper()
	if( $formats -contains $toExtension ) {
		&$scriptBlock
	}
}

function Is-SupportedForFormats ( $file, $supported )
{
	$format = $file._Extension.ToUpper()
	if($supported -notcontains $format)
	{
		throw("The file format '$format' is not supported for your publish! Supported formats are: '$($supported -join ',')'")
	}
}

function Create-Directory([system.io.fileinfo]$localDestFile)
{
	if(! $localDestFile.Directory.Exists ){
		   $localDestFile.Directory.Create()
	}
}

function Remove-File {
	param (
		$file = $(throw "The parameter -file is required.")
    )
	If (Test-Path $file){
		Remove-Item $file -Force
	}
}

#endregion